package com.hxy.study;

import com.hxy.study.config.ElasticSearchConfig;
import com.hxy.study.util.HttpClientHelper;
import io.searchbox.client.JestClient;
import io.searchbox.core.Search;
import io.searchbox.core.SearchResult;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;

import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.HashMap;
import java.util.Map;

@RunWith(SpringRunner.class)
@SpringBootTest
public class SearchClientTest {

    @Autowired
    private ElasticSearchConfig elasticSearchConfig;

    @Autowired
    private HttpClientHelper httpClientHelper;

    @Autowired
    private JestClient jestClient;

    @Test
    public void getInterAddress() {
        try {
            InetAddress inetAddress = InetAddress.getByName(elasticSearchConfig.getServerIp());
            System.out.println();
        } catch (UnknownHostException e) {
            e.printStackTrace();
        }
    }


    /**
     * 使用Jest 利用 Http res接口访问 ElasticSearch
     * 查询条件用的是ElasticSearch官方提供的类
     */
    @Test
    public void testJestEs() {
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        QueryBuilder queryBuilder = QueryBuilders
                .matchQuery("name", "英语试题");//单值完全匹配查询
        QueryBuilder queryBuilderAuthor = QueryBuilders.matchQuery("author", "马");
        QueryBuilder allClause = QueryBuilders.boolQuery().should(queryBuilder).should(queryBuilderAuthor);
        searchSourceBuilder.query(allClause);
        searchSourceBuilder.size(10);
        searchSourceBuilder.from(0);

        String query = searchSourceBuilder.toString();
        System.out.println(query);
        Search search = new Search.Builder(query)
                .addIndex("t_paper")
                .build();
        try {
            SearchResult result = jestClient.execute(search);
            System.out.println(result.getJsonString());
        } catch (IOException e) {
            e.printStackTrace();
        }

        // other examples
//        QueryBuilder levelClause = QueryBuilders.termsQuery("loglevel.keyword", "ERROR", "FATAL");
//        QueryBuilder timeClause = QueryBuilders.rangeQuery("logtime.keyword").from(startTime).to(endTime);
//        QueryBuilder queryBuilder = QueryBuilders.matchQuery("logclass.keyword", "com.huohuo.Test");
        // and/or/not等：涉及到多条件查询用boolQuery，组合多个query。must表示and，mustNot表示not，should表示or。


    }


    @Test
    public void restSearchTest() {
        String url = "http://192.168.232.157:9200/accounts/person/_search";
        Map<String, Object> data = new HashMap<>();

        Map<String, Object> match = new HashMap<>();
        match.put("desc", "软件 系统");

        data.put("query", match);
        try {
            httpClientHelper.get(url, null, (httpstatus, httpMethod, cookies) -> {
                System.out.println(httpstatus);
                System.out.println(httpMethod.getResponseBodyAsString());
            });
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
